import tempfile
import os
from pftp import FTPSession

def parse_recv_resp(resp):
  ln = resp.next()
  while not ln.startswith('----'):
    ln = resp.next()
  #hdr = [h.strip() for h in ln.split('  ')]
  d = {}
  yield "No. SNRF           APRF           St Service Reference    Sender         Size"
  yield "--- -------------- -------------- -- -------------------- -------------- ----"
  no = 0
  for ln in resp:
    if ln.startswith('----'): continue
    v = ln.split()
    no += 1
    yield "%03d %-14s %-14s %2s %-20s %-14s %4s" % (
      no,v[3],v[2],v[1],v[4],v[0],v[5]
    )

def parse_send_resp(resp):
  yield "SNRF           APRF           Sender/Receiver      Status"
  yield "-------------- -------------- -------------------- ----"
  for ln in resp:
    if ln[:3].isdigit(): continue
    d = {}
    for s in ln.split(','):
      name,val = s.split('=',1)
      d[name] = val.rstrip()
    d['sender/receiver'] = "%(sender)s/%(receiver)s" % d
    yield "%(snrf)-14s %(aprf)-14s %(sender/receiver)-20s %(status)s" % d

class PubnetSession(FTPSession):

  def login(self,user=None,passwd=None):
    self.ftp.login('%s@edi.pubnet.org'%self.user,self.passwd)

  def dial(self,host=None):
    FTPSession.dial(self,host)
    self.ftp.login(self.proxyuser,self.proxypasswd)

  def list(self,folder=None):
    return '\n'.join(parse_recv_resp(iter(self.dir(folder))))

  def dnload(self):
    fd,fname = tempfile.mkstemp('','tmp.d.','.')
    fp = os.fdopen(fd,'w+t')
    return self.ftp.retrlines('RETR *',lambda ln: fp.write(ln+'\n'))

  def upload(self,fname,rcpt=None):
    fp = open(fname)
    if rcpt:
      resp = self.ftp.storlines('STOR %s'%rcpt,fp)
    else:
      resp = self.ftp.storlines('STOR %s'%fname,fp)
    fp.close()
    try:	# try interpreting send response
      resp = '\n'.join(parse_send_resp(resp.splitlines()))
    except: pass
    return resp
