import tempfile
import os
import glob
import time
from ediftp import FTPSession
from ftplib import print_line
try:
  import paramiko110 as paramiko
except:
  import paramiko

class SFTPSession(FTPSession):

  def login(self,user=None,passwd=None):
    if user != self._curuser: self.logout()
    self.dial(user=user)

  def logout(self):
    # Close SFTP Connection.
    if self._sftp:
      self._sftp.close()
      self._sftp = None
    # Close the SSH Transport.
    if self._transport:
      self._transport.close()
      self._transport = None

  def list(self,folder=None):
    self.set_state(1)
    if not folder:
      folder = self.get_folder
    if folder:
      self.cwd(folder)
    return '\n'.join(self.listdir())

  def ndir(self,folder=None):
    self.set_state(1)
    if not folder:
      folder = self.get_folder
    self.cwd(folder)
    lns = []
    return '\n'.join(self.listdir())

  def dial(self,host=None,user=None):
    self._sftp = None
    self._curuser = None
    self._transport = None
    if not host and not user:
      return
    if host: 
      self.host = host
    if not user:
      user = self.user
    v = self.host.split(':')
    port = 22
    if v > 1:
      port = int(v[-1])
      host = v[0]
    else:
      host = self.host
    self._transport = paramiko.Transport((host, port))
    private_key = self.private_key
    private_key_pass = None
    if self.passwd:
        self._transport.connect(username = user, password = self.passwd)
    else:
        # Use Private Key.
        if not private_key:
            # Try to use default key.
            if os.path.exists(os.path.expanduser('~/.ssh/id_rsa')):
                private_key = '~/.ssh/id_rsa'
            elif os.path.exists(os.path.expanduser('~/.ssh/id_dsa')):
                private_key = '~/.ssh/id_dsa'
            else:
                raise TypeError, "You have not specified a password or key."
        private_key_file = os.path.expanduser(private_key)
        try:
            xSx_key = paramiko.RSAKey.from_private_key_file(private_key_file,private_key_pass)
        except paramiko.SSHException:
            xSx_key = paramiko.DSSKey.from_private_key_file(private_key_file,password=private_key_pass)
        print 'username =',user
        self._transport.connect(username = user, pkey = xSx_key)
        self._curuser = user

  def retrlines(self, cmd, callback = None):
    if callback is None: callback = print_line
    for ln in self.execute(cmd):
      callback(line)

  def dnload(self):
    self.set_state(1)
    self.cwd(self.get_folder)
    #files = [ln.split()[-1] for ln in self.dir()]
    #for fn in files[1:]:
    files = self.listdir()
    for fn in files:
      #log("fn='"+fn+"'")
      ofn = fn
      seq = 0
      while os.path.exists(ofn):
	seq += 1
	ofn = '%s.%d'%(fn,seq)
      with open(ofn,'w') as fp:
        with self._sftp_connect().open(fn) as rfp:
          for ln in rfp.readlines():
            fp.write(ln+'\n')
	fp.flush()
	os.fsync(fp.fileno())
	self._sftp_connect().remove(fn)

  def upload(self,fpat,rcpt=None,tmp=None):
    self.set_state(2)
    self.cwd(self.put_folder)
    rc = ''
    for fn in glob.glob(fpat):
      with open(fn,'r') as fp:
	if not rcpt:
	  rcpt = os.path.basename(fn)
	if not tmp:
	  tmp = time.strftime('EDIUN_%Y%m%d%H%M%S.tmp')
	else:
	  tmp = time.strftime(tmp)
        stat = self._sftp_connect().putfo(fp, tmp, confirm=False)
	rc += str(stat)
      if False and tmp != rcpt:
        # Amazon eats uploaded files
	resp = self.rename(tmp,rcpt)
	rc += resp
    return rc

  def cwd(self, path):
    """change the current working directory on the remote"""
    self._sftp_connect().chdir(path)

  def _sftp_connect(self):
    """Establish the SFTP connection."""
    if not self._sftp:
      self._sftp = paramiko.SFTPClient.from_transport(self._transport)
    return self._sftp

  def listdir(self, path='.'):
      """return a list of files for the given path"""
      return self._sftp_connect().listdir(path)

  def rename(self, oldpath, newpath):
      """ Re-name old name to New name """
      self._sftp_connect().rename(oldpath, newpath)

  def execute(self, command):
    """Execute the given commands on a remote machine."""
    channel = self._transport.open_session()
    channel.exec_command(command)
    output = channel.makefile('rb', -1).readlines()
    if output:
      return output
    return channel.makefile_stderr('rb', -1).readlines()
