#!/usr/bin/python2

from ftplib import FTP,error_perm
import sys
import time
import tempfile
import os
import glob
import logging 
import smtplib
from ConfigParser import SafeConfigParser as ConfigParser

logging.basicConfig(
        stream=sys.stdout,
        level=logging.INFO,
        format='%(asctime)s %(message)s',
        datefmt='%Y%b%d %H:%M:%S'
)

pubnet_log = logging.getLogger('pubnet')

def log(*msg):
  pubnet_log.info(*msg)

class FTPSession(object):

  def __init__(self, host=None, user=None, passwd=None,
  	getdir=None, putdir=None, putuser=None, putpasswd=None,
        proxyuser=None, proxypasswd=None, private_key=None):
    self.host = host
    self.user = user
    self.passwd = passwd
    if putuser:
      self.putuser = putuser
    else:
      self.putuser = user
    if putpasswd:
      self.putpasswd = putpasswd
    else:
      self.putpasswd = passwd
    if proxyuser:
      self.proxyuser = proxyuser
    else:
      self.proxyuser = user
    if proxypasswd:
      self.proxypasswd = proxypasswd
    else:
      self.proxypasswd = passwd
    self.private_key = private_key
    self.state = 0
    self.get_folder = getdir
    self.put_folder = putdir
    dir(self)
    
  def dial(self,host=None):
    if host: self.host = host
    self.ftp = FTP(self.host)

  def login(self,user=None,passwd=None):
    self.ftp.login(user,passwd)

  def logout(self):
    self.ftp.quit()

  # state 0 - login
  # state 1 - download
  # state 2 - upload
  def set_state(self,state):
    if state != self.state:
      if state == 1:
        self.login(self.user,self.passwd)
      elif state == 2:
        if self.putuser != self.user:
	  self.login(self.putuser,self.putpasswd)
	else:
	  self.login(self.user,self.passwd)
      self.state = state
    
  def dir(self,folder=None):
    self.set_state(1)
    if not folder:
      folder = self.get_folder
    if folder:
      self.ftp.cwd(folder)
    lns = []
    self.ftp.retrlines('LIST',lambda ln: lns.append(ln))
    return lns

  def ndir(self,folder=None):
    self.set_state(1)
    if not folder:
      folder = self.get_folder
    self.ftp.cwd(folder)
    lns = []
    self.ftp.retrlines('NLST',lambda ln: lns.append(ln))
    return lns

  def list(self,dir=None):
    return '\n'.join(self.dir(dir))

  def dnload(self):
    self.set_state(1)
    self.ftp.cwd(self.get_folder)
    #files = [ln.split()[-1] for ln in self.dir()]
    #for fn in files[1:]:
    files = self.ndir()
    for fn in files:
      #log("fn='"+fn+"'")
      ofn = fn
      seq = 0
      while os.path.exists(ofn):
	seq += 1
	ofn = '%s.%d'%(fn,seq)
      fp = open(ofn,'w')
      try:
	self.ftp.retrlines('RETR %s'%fn,lambda ln: fp.write(ln+'\n'))
	fp.flush()
	os.fsync(fp.fileno())
	self.ftp.delete(fn)
      finally: fp.close()

  def upload(self,fpat,rcpt=None,tmp=None):
    self.set_state(2)
    self.ftp.cwd(self.put_folder)
    rc = ''
    for fn in glob.glob(fpat):
      fp = open(fn)
      try:
	if not rcpt:
	  rcpt = os.path.basename(fn)
	if not tmp:
	  tmp = time.strftime('EDIUN_%Y%m%d%H%M%S.tmp')
	else:
	  tmp = time.strftime(tmp)
	resp = self.ftp.storlines('STOR %s'%tmp,fp)
	rc += resp
      finally: fp.close()
      if tmp != rcpt:
	resp = self.ftp.rename(tmp,rcpt)
	rc += resp
    return rc

def mailmsg(txt):
  fromaddr = 'stuart@editorialunilit.com'
  toaddrs = ('stuart@bmsi.com','sil@editorialunilit.com')
  msg = '\r\n'.join((
    'From: %s'%fromaddr,
    'To: %s'%', '.join(toaddrs),
    'Subject: %s',txt,
    '',
    'Please investigate.'
  ))
  mta = smtplib.SMTP('localhost')
  mta.sendmail(fromaddr,toaddrs,msg)
  mta.quit()

def create_session(config):
  cp = ConfigParser()
  s = cp.read([os.path.expanduser('~/.ediftp.cfg'),'/bms/edi/ediftp.cfg'])
  cf = dict(cp.items(config))
  sesstype = 'FTP'
  m = __name__
  if 'type' in cf:
    sesstype = cf['type']
    #m = 'ediftp.'+sesstype
    module = __import__(sesstype,globals(),locals())
    del cf['type']
  else:
    module = sys.modules[m]
  try:
    klass = getattr(module,sesstype+'Session')
  except:
    print module.__name__,dir(module)
  return klass(**cf)
